<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Indexing;

class Indexing
{

    public function __construct()
    {
    }

    public function index(array $eventTypes){
        foreach($eventTypes as $eventType){
            $this->indexEventType($eventType);
        }
    }

    public function indexEventType(\PX_NeedToDoEventTypes $eventType){
        if(!$this->eventTypeHasValidTargetData($eventType)){
            return;
        }
        $targetRecords = $this->getTargetRecords($eventType);
        $this->indexTargetRecords($targetRecords, $eventType);
//        $this->updateLastProcessedDateInEventType($eventType);

    }

    private function eventTypeHasValidTargetData(\PX_NeedToDoEventTypes $eventType){
        return !empty($eventType->target_module) && !empty($eventType->target_attribute_name);
    }

    private function getTargetRecords(\PX_NeedToDoEventTypes $eventType){
        $targetModule = $eventType->target_module;
        $targetAttributeName = $eventType->target_attribute_name;
        $targetBean = \BeanFactory::getBean($targetModule);
        $targetRecords = array();

        if(empty($targetBean)) {
            return $targetRecords;
        }

        $sugarQuery = new \SugarQuery();
        $sugarQuery->from($targetBean);
        $sugarQuery->select(array('id', $targetAttributeName));
        $sugarQuery->where()->isNotEmpty($targetAttributeName);

        $filterDefs = array();
        if(!empty($eventType->target_entity_filter) && is_string($eventType->target_entity_filter)){
            $filterDefs = json_decode($eventType->target_entity_filter, true);
        }
        if(!empty($filterDefs)) {
            require_once 'modules/PX_NeedToDoEventTypes/clients/base/api/PX_NeedToDoEventTypesFilterApi.php';
            $needToDoEventTypesFilterApi = new \PX_NeedToDoEventTypesFilterApi();
            $needToDoEventTypesFilterApi->addTargetFilters($filterDefs, $sugarQuery->where(), $sugarQuery);
        }


        //TODO: Remove last_processed_date!!!
//        if(!empty($eventType->last_processed_date)){
//            $sugarQuery->where()->gt('date_modified', $eventType->last_processed_date);
//        }

        $queryRows = $sugarQuery->execute();
        foreach($queryRows as $row){
            $targetRecords[] = array(
                'object_type' => $targetModule,
                'object_id' => $row['id'],
                'control_date' => $row[$targetAttributeName],
                'control_date_type' => $eventType->target_attribute_type,
                'event_type_id' => $eventType->id,
            );
        }

        return $targetRecords;
    }

    private function indexTargetRecords($targetRecords, $eventType){
        foreach($targetRecords as $targetRecord){
            $this->indexTargetRecord($targetRecord, $eventType);
        }
    }

    private function indexTargetRecord($targetRecord, $eventType){
        $needToDoEventId = $this->getNeedToDoEventIdByAttributes($targetRecord);

        $needToDoEventBean = \BeanFactory::getBean('PX_NeedToDoEvents', $needToDoEventId);
        $needToDoEventBean->control_date = $targetRecord['control_date'];
        $needToDoEventBean->control_date_type = $targetRecord['control_date_type'];
        $needToDoEventBean->object_type = $targetRecord['object_type'];
        $needToDoEventBean->object_id = $targetRecord['object_id'];
        $needToDoEventBean->setNeedToDoEventTypeBean($eventType);
        $needToDoEventBean->event_type_id = $targetRecord['event_type_id'];
        $needToDoEventBean->save();
    }

    private function getNeedToDoEventIdByAttributes(array $targetRecord){
        $sugarQuery = new \SugarQuery();
        $sugarQuery->from(\BeanFactory::getBean('PX_NeedToDoEvents'));
        $sugarQuery->select(array('id'));
        $sugarQuery->where()->equals('object_type', $targetRecord['object_type']);
        $sugarQuery->where()->equals('object_id', $targetRecord['object_id']);
        $sugarQuery->where()->equals('event_type_id', $targetRecord['event_type_id']);
        $queryResult = $sugarQuery->execute();

        $needToDoEventId = null;
        if(!empty($queryResult) && count($queryResult) > 0){
            $needToDoEventId = $queryResult[0]['id'];

            $idsToRemove = array();
            foreach($queryResult as $key => $row){
                if(!$key) continue;
                $idsToRemove[] = $row['id'];
            }
            $this->removeOldNeedToDoEvents($idsToRemove);
        }

        return $needToDoEventId;
    }

    private function removeOldNeedToDoEvents(array $idsToRemove){
        if(empty($idsToRemove))
            return;

        $sql = "DELETE FROM px_needtodoevents WHERE id in ('" . implode("', '", $idsToRemove) . "')";
        $db = \DBManagerFactory::getInstance();
        $db->query($sql);
    }

    private function updateLastProcessedDateInEventType(\PX_NeedToDoEventTypes $eventType){
        global $timedate;
        $eventType->last_processed_date = $timedate->nowDb();
        $eventType->save();
    }
}