<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\PX_NeedToDoViewerComments\Helpers;

class ConversationParticipants{

    /**
     * @var \PX_NeedToDoViewerComments
     */
    protected $commentBean;

    protected $needToDoEventBean;
    protected $contextBean;
    protected $mainCommentBean;


    /**
     * @var array
     */
    protected $usersIds = array();

    /**
     * @var string
     */
    protected $messageAuthorId = '';

    /**
     * @var string
     */
    protected $dialogAuthorId = '';

    /**
     * @var array
     */
    protected $mentionedUsersIds = array();

    /**
     * @var array
     */
    protected $mentionedPreviouslyUsersIds = array();

    /**
     * @var array
     */
    protected $participantsIds = array();

    /**
     * @var array
     */
    protected $relevantIds = array();

    /**
     * @var array
     */
    protected $subscribedIds = array();


    /**
     * ConversationParticipants constructor.
     * @param \PX_NeedToDoViewerComments $commentBean
     */
    public function __construct(\PX_NeedToDoViewerComments $commentBean){
        $this->commentBean = $commentBean;

        $this->init();
    }

    protected function init(){
        $this->setMentionedUsersIds();
        $this->setMessageAuthorId();
        $this->setDialogAuthorId();
        $this->setParticipantIds();
        $this->setSubscribedIds();
    }

    protected function setMentionedUsersIds(){
        $commentData = $this->commentBean->getCommentDataAsArray();
        if (isset($commentData['tags'])) {
            foreach ($commentData['tags'] as $tag) {
                if ($tag['module'] === 'Users') {
//                    $userId = $tag['id'];
//                    if($this->userHasViewAccess($userId)) {
                        $this->addUserToMentioned($tag['id']);
//                    }
                }
            }
        }
    }

    protected function userHasViewAccess($userId)
    {
        $access = true;

		require_once 'modules/ACLActions/ACLAction.php';
        $access = ACLAction::userHasAccess($userId, 'PX_NeedToDoEvents', 'view');

        if($access){
            $contextModuleName = $this->getContextModuleName();
            if(!empty($contextModuleName))
                $access = ACLAction::userHasAccess($userId, $contextModuleName, 'view');
        }

        return $access;
    }

    public function getNeedToDoEventBean(){
        if(!isset($this->needToDoEventBean)) {
            $this->needToDoEventBean = $this->setNeedToDoEventBean($this->commentBean);
        }
        return $this->needToDoEventBean;
    }

    private function setNeedToDoEventBean($comment, $iteration = 0){
        $parent = $comment->getParentBean();
        if(empty($parent))
            return null;
        else if($parent->module_name == 'PX_NeedToDoEvents'){
            return $parent;
        }
        else if($iteration >= 5)
            return null;

        return $this->setNeedToDoEventBean($parent, $iteration + 1);
    }

    public function getContextModuleName(){
        global $moduleList;
        $needToDoEventBean = $this->getNeedToDoEventBean($this->commentBean);

        if(!empty($needToDoEventBean) && !empty($needToDoEventBean->parent_type) && in_array($moduleList, $needToDoEventBean->parent_type)){
            return  $needToDoEventBean->parent_type;
        }

        return '';
    }

    public function getContextBean(){
        if(!isset($this->contextBean)){
            $this->setContextBean();
        }

        return $this->contextBean;
    }

    public function setContextBean(){
        $needToDoEventBean = $this->getNeedToDoEventBean($this->commentBean);
        if(!empty($needToDoEventBean)){
            $this->contextBean = \BeanFactory::retrieveBean($needToDoEventBean->parent_type, $needToDoEventBean->parent_id);
        }else
            $this->contextBean = null;
    }

    public function addUserToMentioned($userId){
        $this->mentionedUsersIds[$userId] = $userId;
    }

    public function setMessageAuthorId($userId = ''){
        if(!empty($userId)){
            $this->messageAuthorId = $userId;
        }else {
//            $mainMessageBean = $this->getMainMessageBean($this->commentBean);
            if (!empty($this->commentBean) && !empty($this->commentBean->created_by)) {
                $this->messageAuthorId = $this->commentBean->created_by;
            }else{
                global $current_user;
                $this->messageAuthorId = $current_user->id;
            }
        }
    }

    public function setDialogAuthorId($userId = ''){
        if(!empty($userId)){
            $this->dialogAuthorId = $userId;
        }else {
            $mainMessageBean = $this->getMainMessageBean($this->commentBean);
            if(!empty($mainMessageBean)) {
                $this->dialogAuthorId = $mainMessageBean->created_by;
            }
        }
    }

    public function getMessageAuthorId(){
        return $this->messageAuthorId;
    }

    public function setParticipantIds(){
        $mainMessage = $this->getMainMessageBean();

        if(empty($mainMessage) || !$mainMessage->load_relationship('dialog_users'))
            return array();

        $query = new \SugarQuery();
        $query->from($mainMessage);
        $messageUsers = $query->join('comments_users', array('joinType' => 'INNER'))->joinName();
        $messageUsersM2M = $messageUsers . '_m2m';
        $query->where()->equals($messageUsersM2M . '.parent_id', $mainMessage->id);
        $query->where()->in($messageUsersM2M . '.type', array('mentioned', 'author', 'participant'));
        $query->select()->addField($messageUsersM2M . '.user_id', array('alias' => 'participant_id'));
        $query->select()->addField($messageUsersM2M . '.type', array('alias' => 'participation_type'));
        foreach($query->execute() as $row){
            $userId = $row['participant_id'];
            if($row['participation_type'] == 'mentioned') {
                $this->mentionedPreviouslyUsersIds[$userId] = $userId;
            }elseif($row['participation_type'] == 'author'){
                $this->participantsIds[$userId] = $userId;
            }
            $this->relevantIds[$userId] = $userId;
        }
    }

    public function setSubscribedIds(){
        $topicComment = $this->getMainMessageBean();
        $query = new \SugarQuery();
        $query->from($topicComment);
        $subscribedJTA = 'subscribed_users';
        $subscribedTableSQL = "(SELECT id, created_by, parent_type, parent_id from subscriptions WHERE deleted=0)";
        $query->join[$subscribedJTA] = $query->joinTable($subscribedTableSQL, array('alias' => $subscribedJTA, 'joinType' => 'LEFT'));
        $query->join[$subscribedJTA]->on()->equalsField($subscribedJTA . '.parent_id', 'px_needtodoviewercomments.id')
            ->equals($subscribedJTA . '.parent_type', 'PX_NeedToDoViewerComments');
        $query->select()->addField($subscribedJTA.'.created_by', array('alias' => 'subscribed_user_id'));
        $query->where()->equals('id', $topicComment->id);
        foreach($query->execute() as $row)
            $this->subscribedIds[$row['subscribed_user_id']] = $row['subscribed_user_id'];
//        $mainMessage = $this->getMainMessageBean();
//        if(empty($mainMessage) || !$mainMessage->load_relationship('following_link'))
//            return array();
//
//        foreach($mainMessage->following_link->get() as $id)
//            $this->subscribedIds[$id] = $id;
    }


    public function setMainMessageBean(\SugarBean $comment){
        if(empty($comment) || !($comment instanceof \PX_NeedToDoViewerComments)){
            $this->mainCommentBean = null;
            return;
        }
        if($comment->parent_type == 'PX_NeedToDoEvents'){
            $this->mainCommentBean = $comment;
            return;
        }
        $comment = $comment->getParentBean();

        $this->setMainMessageBean($comment);
    }

    public function getMainMessageBean(){
        if(!isset($this->mainCommentBean)){
            $this->setMainMessageBean($this->commentBean);
        }
        return $this->mainCommentBean;
    }

//    public function getAllParticipants(){
//        $userIds = array_merge($this->mentionedUsersIds, $this->relevantIds);
//        $userIds[$this->messageAuthorId] = $this->messageAuthorId;
//        return $userIds;
//    }

    public function getMentionedUsersIds(){
        return $this->mentionedUsersIds;
    }

    public function getMentionedPreviouslyUsersIds(){
        return $this->mentionedPreviouslyUsersIds;
    }

    public function getRelevantIds(){
        return $this->relevantIds;
    }

    public function getParticipantsIds(){
        return $this->participantsIds;
    }

    public function getSubscribedIds(){
        return $this->subscribedIds;
    }

    public function getDialogAuthorIds(){
        return $this->dialogAuthorId;
    }

    public function getClippedMentionedUsersIds(){
        return array_diff($this->mentionedUsersIds, array($this->messageAuthorId => $this->messageAuthorId));
    }

    public function getClippedParticipantUsersId(){
        return array_diff($this->relevantIds, $this->mentionedUsersIds, array($this->messageAuthorId => $this->messageAuthorId));
    }

    public function getClippedSubscribedUsersId(){
        return array_diff($this->subscribedIds, $this->relevantIds, $this->mentionedUsersIds, array($this->messageAuthorId => $this->messageAuthorId));
    }


}