({
    extendsFrom: "RecordlistView",

    initialize: function (options) {
        options.meta.type = "recordlist";
        this._super('initialize', [options]);

        this.context.on("list:revoke:fire", this.revokeAccessClicked, this);
        this.context.parent.on('users:add', this.grantAccessClicked, this);
        this.context.parent.on('users:reload', function () {
            this.collection.fetch();
        }, this);

        this.collection.filterDef = {
            '$and': [
                {'px_ntdv_access_c': 1},
                {'$or': [
                        {'is_group': 0},
                        {'is_group': {'$is_null': 1}}
                    ]},
                {'$or': [
                        {'portal_only': 0},
                        {'portal_only': {'$is_null': 1}}
                    ]}
            ],
            status: 'Active'
        };
    },

    chunkModelIds: function (models) {
        var ids = _.map(models, function (model) {
            return model.id;
        });

        if(_.indexOf(ids, app.user.id) >=0){
            ids = _.filter(ids, function (id) {
                return id !== app.user.id;
            });
            ids.push(app.user.id);
        }

        var chunks = [],
            chunkSize = 50;
        while (ids.length) {
            chunks.push(ids.splice(0, chunkSize))
        }

        return chunks;
    },

    getUserIds: function (models) {
        return _.map(models, function (model) {
            return model.id;
        });
    },

    grantAccessClicked: function () {
        var context = {
            module: 'Users',
            isMultiSelect: true,
            filterOptions: new app.utils.FilterOptions()
                .config({
                    initial_filter: 'px_need_to_do_users_with_no_access',
                    initial_filter_label: 'LBL_PX_TIMELINE_USERS_WITH_NO_ACCESS_FILTER',
                    filter_populate: {
                        '$and': [
                            {'$or': [
                                    {'px_ntdv_access_c': 0},
                                    {'px_ntdv_access_c': {'$is_null': 1}}
                                ]},
                            {'$or': [
                                    {'is_group': 0},
                                    {'is_group': {'$is_null': 1}}
                                ]},
                            {'$or': [
                                    {'portal_only': 0},
                                    {'portal_only': {'$is_null': 1}}
                                ]}
                        ],
                        status: 'Active'
                    }
                }).format()
        };

        app.drawer.open({
            layout: 'multi-selection-list',
            context: context
        }, _.bind(this.usersSelected, this));
    },

    grantUserAccess: function (response) {
        response = response || {};
        response['addedUsers'] = (response['addedUsers'] || 0);
        response['errorMessage'] = response['errorLabel'] || '';

        app.alert.dismiss("ntdv_license_granting_access_to_users");
        this.collection.fetch();
        this.context.parent.loadData();

        if (!_.isEmpty(response['errorMessage'])) {
            if(response['addedUsers']) {
                app.alert.show('ntdv_license_revoking_access_from_users_success', {
                    level: 'success',
                    messages: app.lang.get('LBL_SUCCESSFUL_ACCESS_GRANTING_TO', 'PX_NeedToDoEvents').replace("%d", response['addedUsers']),
                    autoClose: true,
                });
            }
            app.alert.show("error", {
                level: "error",
                messages: response['errorMessage'],
                autoClose: true,
                autoCloseDelay: 8000
            });
        }else{
            app.alert.show('ntdv_license_revoking_access_from_users_success', {
                level: 'success',
                messages: app.lang.get('LBL_SUCCESSFUL_ACCESS_GRANTING', 'PX_NeedToDoEvents'),
                autoClose: true,
            });
        }
    },

    usersSelected: function (models) {
        var chunks = this.chunkModelIds(models);

        var tasks = [];

        _.each(chunks, function (chunk) {
            tasks.push(_.bind(function(arg1, callback) {
                callback = _.isFunction(arg1) && _.isEmpty(callback) ? arg1 : callback;

                var url = app.api.buildURL('PX_NeedToDoEvents/license/grant-user-access');
                app.api.call('update', url, { ids: chunk}, {
                    success: _.bind(function (response) {
                        callback(null, response);
                    }, this),
                    error: _.bind(function (error) {
                        callback(true, {errorMessage: error.message});
                    }, this)
                });
            }, this));
        }, this);

        app.alert.show('ntdv_license_granting_access_to_users', {
            level: 'process',
            title: app.lang.get('LBL_LOADING'),
            autoClose: false
        });


        async.waterfall(tasks, _.bind(this.grantUserAccessResponse, this));
    },

    revokeAccessClicked: function (model) {
        var models = [];

        if (this.context.get('mass_collection').length)
            models = this.context.get('mass_collection').models;
        else
            models = [model];

        var chunks = this.chunkModelIds(models);
        var errorMessage = '';

        var tasks = [];

        _.each(chunks, function (chunk) {
            tasks.push(function(arg1, callback) {
                callback = _.isFunction(arg1) && _.isEmpty(callback) ? arg1 : callback;

                var url = app.api.buildURL('PX_NeedToDoEvents/license/revoke-user-access');
                app.api.call('update', url, { ids: chunk }, {
                    success: _.bind(function (response) {
                        callback(null, response);
                    }, this),
                    error: function (error) {
                        callback(true, {errorLabel: error.message});
                    }
                });
            });
        });

        app.alert.show('ntdv_license_revoking_access_from_users', {
            level: 'process',
            title: app.lang.get('LBL_LOADING'),
            autoClose: false
        });

        async.waterfall(tasks, _.bind(this.revokeUserAccessResponse, this));
    },

    revokeUserAccessResponse: function(tmp, response){
        response = response || {};
        var errorMessage = response['errorLabel'] || '',
            activeUsers = response['activeUsers'] || '',
            config = response['config'] || '';

        app.alert.dismiss("ntdv_license_revoking_access_from_users");
        this.collection.fetch();
        this.context.parent.trigger("settings:update", config);

        if (!_.isEmpty(errorMessage)) {
            app.alert.show("ntdv_license_revoking_access_from_users_error", {
                level: "error",
                messages: errorMessage
            });
        }else{
            app.alert.show('ntdv_license_revoking_access_from_users_success', {
                level: 'success',
                messages: app.lang.get('LBL_SUCCESSFUL_ACCESS_REVOKING', 'PX_NeedToDoEvents'),
                autoClose: true
            });
        }
    },

    // helperReverseArray: function(a) {
    //     var temp = [],
    //         len = a.length;
    //     for (var i = (len - 1); i !== -1; i--) {
    //         temp.push(a[i]);
    //     }
    //     return temp;
    // },

    grantUserAccessResponse: function (tmp, response) {
        response = response || {};
        var grantedAccessTo = (response['addedUsers'] || 0),
            errorMessage = response['errorLabel'] || '',
            activeUsers = response['activeUsers'] || '',
            config = response['config'] || '';

        app.alert.dismiss("ntdv_license_granting_access_to_users");
        this.collection.fetch();
        this.context.parent.trigger("settings:update", config);

        if (!_.isEmpty(errorMessage)) {
            if(grantedAccessTo) {
                app.alert.show('ntdv_license_granting_access_from_users_success', {
                    level: 'success',
                    messages: app.lang.get('LBL_SUCCESSFUL_ACCESS_GRANTING_TO', 'PX_NeedToDoEvents').replace("%d", grantedAccessTo),
                    autoClose: true,
                });
            }
            app.alert.show("ntdv_license_granting_access_from_users_error", {
                level: "error",
                messages: errorMessage,
                autoClose: true,
                autoCloseDelay: 8000
            });
        }else if(!_.isEmpty(response['status']) && response.status == 'success'){
            app.alert.show('ntdv_license_granting_access_from_users_success', {
                level: 'success',
                messages: app.lang.get('LBL_SUCCESSFUL_ACCESS_GRANTING', 'PX_NeedToDoEvents'),
                autoClose: true,
            });
        }
    },

    // render: function () {
    //     this._super("render");
    //
    //     // we need to set the viewName to list for all row action fields here because of a bug in 7.6 & 7.7
    //     _.each(this.fields, function (field) {
    //         if (field.model.id && field.type === "fieldset") {
    //             field.setViewName("list");
    //             field.render();
    //         }
    //     });
    // }
})
