<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

require_once 'clients/base/api/FileApi.php';

class PX_NeedToDoEventTypesFileApi extends FileApi {
    /**
     * Dictionary registration method, called when the API definition is built
     *
     * @return array
     */
    public function registerApiRest() {
        return array(
            'getFileContents' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEventTypes', '?', 'file', '?'),
                'pathVars' => array('module', 'record', '', 'field'),
                'method' => 'getFile',
                'rawReply' => true,
                'allowDownloadCookie' => true,
                'shortHelp' => 'Gets the contents of a single file related to a field for a module record.',
                'longHelp' => 'include/api/help/module_record_file_field_get_help.html',
            ),
            'removeFile' => array(
                'reqType' => 'DELETE',
                'path' => array('PX_NeedToDoEventTypes', '?', 'file', '?'),
                'pathVars' => array('module', 'record', '', 'field'),
                'method' => 'removeFile',
                'rawPostContents' => true,
                'shortHelp' => 'Removes a file from a field.',
                'longHelp' => 'include/api/help/module_record_file_field_delete_help.html',
            ),
        );
    }

    public function getFile(ServiceBase $api, array $args){
        // if exists link_name param then get archive
        if (!empty($args['link_name'])) {
            // @TODO Remove this code and use getArchive method via rest
            $this->getArchive($api, $args);
            return;
        }

        // Get the field
        if (empty($args['field'])) {
            // @TODO Localize this exception message
            throw new SugarApiExceptionMissingParameter('Field name is missing');
        }
        $field = $args['field'];

        // Get the bean
        $bean = $this->loadBean($api, $args);

        if(!$bean->ACLAccess('view')) {
            throw new SugarApiExceptionNotAuthorized('No access to view records for module: '.$args['module']);
        }

        if (empty($bean->{$field})) {
            // @TODO Localize this exception message
            throw new SugarApiExceptionNotFound("The requested file $args[module] :: $field could not be found.");
        }

        // Handle ACL
        $this->verifyFieldAccess($bean, $field);

        $def = $bean->field_defs[$field];
        //for the image type field, forceDownload set default as false in order to display on the image element.
        $forceDownload = ($def['type'] == 'image') ? false : true;
        if (isset($args['force_download'])) {
            $forceDownload = (bool) $args['force_download'];
        }

        $download = $this->getDownloadFileApi($api);
        try {
            return $download->getFile($bean, $field, $forceDownload);
        } catch (Exception $e) {
            throw new SugarApiExceptionNotFound($e->getMessage(), null, null, 0, $e);
        }
    }


    protected function getDownloadFileApi(ServiceBase $api)
    {
        require_once 'modules/PX_NeedToDoEventTypes/clients/base/api/helper/PX_NeedToDoEventTypesDownloadFileApi.php';
        return new PX_NeedToDoEventTypesDownloadFileApi($api);
    }

    protected function loadBean(ServiceBase $api, array $args, $aclToCheck = 'view', array $options = array()){
        $bean = parent::loadBean($api, $args, $aclToCheck, $options);

        if(!empty($bean) && !empty($args['field']) && $args['field'] == 'icon')
            $bean->field_defs[$args['field']]['type'] = 'image';

        return $bean;
    }

}
