({

    ntdvLicenseModel: null,

    initialize: function (options) {
        this._super("initialize", [options]);

        this.context.on('settings:save', this.saveNtdvLicense, this);
        this.context.on('settings:reload', this.loadData, this);
        this.context.on('settings:update', this.updateNtdvLicense, this);

        this.ntdvLicenseModel = new app.data.beanModel();

        this.ntdvLicenseModel.on('change:license_type', this.toggleUsersPanel, this);
        this.ntdvLicenseModel.on('change:users_limit change:active_users', this.checkAddUsersButton, this);
    },

    saveNtdvLicense: function () {
        var data = {};

        _.each(this.meta.fields, function (def) {
            data[def.name] = this.ntdvLicenseModel.get(def.name);
        }, this);

        app.alert.show('settings:save', {
            level: 'process',
            title: app.lang.getAppString('LBL_LOADING')
        });

        var url = app.api.buildURL(this.module, 'config');

        app.api.call('update', url, data, {
            success: _.bind(function (data) {
                app.alert.show('settings:success', {
                    level: 'success',
                    title: app.lang.getAppString('LBL_SUCCESS'),
                    messages: app.lang.get('LBL_LICENSE_TOKEN_SUCCESSFULLY_SAVED', this.module),
                    autoClose: true
                });

                this.setLicenseLicense(data);
                this.render();
                if(data['license_type'] !== 'Demo')
                    this.context.trigger("users:reload");
            }, this),
            error: _.bind(function (error) {
                if (error.code === "ntdv_invalid_license") {
                    app.alert.show('ntdv_license_save_success', {
                        level: 'success',
                        title: app.lang.getAppString('LBL_SUCCESS'),
                        messages: app.lang.get('LBL_LICENSE_TOKEN_SUCCESSFULLY_SAVED', this.module),
                        autoClose: true
                    });
                    app.alert.show('ntdv_license_is_invalid', {
                        level: 'error',
                        messages: error.message,
                    });
                } else {
                    app.alert.show('ntdv_license_is_invalid', {
                        level: 'error',
                        messages: 'Error when saving license: ' + app.lang.getAppString(error.message),
                    });
                }

                this.loadData();
            }, this),
            complete: _.bind(function (data) {
                app.alert.dismiss('settings:save');
            }, this)
        });
    },

    setLicenseLicense: function (data) {
        if (!_.isEmpty(data)) {
            _.each(data, function (value, key) {
                if(key == 'expiry_date'){
                    value = app.date(value);
                    if (value.isValid()) {
                        value = value.formatUser(true);
                    }else{
                        value = '';
                    }
                }
                this.ntdvLicenseModel.set(key, value);
            }, this);
        }
    },

    updateNtdvLicense: function(data){
        this.setLicenseLicense(data);
        this.render();
    },


    loadData: function () {
        app.api.call('read', app.api.buildURL(this.module, 'config'), {}, {
            success: _.bind(function (data) {
                this.setLicenseLicense(data);
                this.render();
            }, this),
            error: _.bind(function (error) {
                var licenseToken = this.ntdvLicenseModel.get('license_token');
                if(!_.isEmpty(licenseToken))
                    app.alert.show("ntdv_license_is_invalid", {
                        level: "error",
                        messages: error.message
                    });
            }, this)
        });
    },

    _renderHtml: function(ctx, options) {
        this._super("_renderHtml", [ctx, options]);
        this.layout.initPanelStates();
    },

    toggleUsersPanel: function(model){
        var licenseType = model.get('license_type');

        if(licenseType == 'Demo'){
            this.context.trigger("users:hidePanel");
        }else{
            this.context.trigger("users:showPanel");
        }
    },

    checkAddUsersButton: function(){
        var limitUsers = this.ntdvLicenseModel.get('users_limit'),
            activeUsers = this.ntdvLicenseModel.get('active_users');

        if(!limitUsers || limitUsers <= activeUsers){
            this.context.trigger("users:disableAddButton");
        }else{
            this.context.trigger("users:enableAddButton");
        }
    }
})
